Package: LaMa
Type: Package
Title: Fast Numerical Maximum Likelihood Estimation for Latent Markov
        Models
Version: 2.1.1
Authors@R: c(
    person(given = "Jan-Ole", family = "Fischer", 
          email = "jan-ole.fischer@mailbox.org",
          role = c("aut", "cre"),
          comment = c(ORCID = "0009-0004-1556-9053")))
Description: 
  A variety of latent Markov models, including hidden Markov models, hidden semi-Markov models, 
  state-space models and continuous-time variants can be formulated and estimated within the same framework via directly maximising the likelihood function using the so-called forward algorithm. 
  Applied researchers often need custom models that standard software does not easily support. 
  Writing tailored 'R' code offers flexibility but suffers from slow estimation speeds. 
  We address these issues by providing easy-to-use functions (written in 'C++' for speed) for common tasks like the forward algorithm. 
  These functions can be combined into custom models in a Lego-type approach, offering up to 10-20 times faster estimation via standard numerical optimisers. 
  To aid in building fully custom likelihood functions, several vignettes are included that show how to simulate data from and estimate all the above model classes.
URL: https://janolefi.github.io/LaMa/
License: MIT + file LICENSE
Encoding: UTF-8
Imports: Rcpp, stats, utils, methods, Matrix, splines2, mgcv, MASS,
        numDeriv, RTMBdist (>= 1.0.4)
LinkingTo: Rcpp, RcppArmadillo
Depends: R (>= 3.5.0), RTMB (>= 1.9.0)
RoxygenNote: 7.3.3
Suggests: knitr, rmarkdown, testthat (>= 3.0.0), PHSMM, MSwM, scales
VignetteBuilder: knitr
Config/testthat/edition: 3
LazyData: true
NeedsCompilation: yes
Packaged: 2026-05-13 09:25:24 UTC; jan-ole
Author: Jan-Ole Fischer [aut, cre] (ORCID:
    <https://orcid.org/0009-0004-1556-9053>)
Maintainer: Jan-Ole Fischer <jan-ole.fischer@mailbox.org>
Repository: CRAN
Date/Publication: 2026-05-13 09:50:02 UTC
Built: R 4.5.2; x86_64-apple-darwin20; 2026-05-13 11:17:58 UTC; unix
Archs: LaMa.so.dSYM
