# Generated by roxygen2: do not edit by hand

S3method(logLik,LaMaModel)
S3method(logLik,qremlModel)
S3method(plot,LaMaResiduals)
S3method(predict,LaMa_matrices)
S3method(summary,qremlModel)
export("%sp%")
export(LaMaColors)
export(MCreport)
export(calc_trackInd)
export(cosinor)
export(ddwell)
export(dgamma2)
export(dgmrf2)
export(dskewnorm)
export(dvm)
export(dwrpcauchy)
export(forward)
export(forward_g)
export(forward_hsmm)
export(forward_ihsmm)
export(forward_p)
export(forward_phsmm)
export(forward_s)
export(forward_sp)
export(generator)
export(generator_g)
export(make_matrices)
export(make_matrices_dens)
export(make_matrices_old)
export(max0_smooth)
export(max2)
export(min0_smooth)
export(min2)
export(penalty)
export(penalty2)
export(penalty_uni)
export(pgamma2)
export(pred_matrix)
export(process_hid_formulas)
export(pseudo_res)
export(pskewnorm)
export(pvm)
export(qgamma2)
export(qreml)
export(qreml_old)
export(qskewnorm)
export(report)
export(rgamma2)
export(rskewnorm)
export(rvm)
export(rwrpcauchy)
export(sdreportMC)
export(sdreport_outer)
export(smooth_dens_construct)
export(stateprobs)
export(stateprobs_g)
export(stateprobs_p)
export(stationary)
export(stationary_cont)
export(stationary_ct)
export(stationary_p)
export(stationary_p_sparse)
export(stationary_sparse)
export(tpm)
export(tpm_cont)
export(tpm_ct)
export(tpm_emb)
export(tpm_emb_g)
export(tpm_g)
export(tpm_g2)
export(tpm_hsmm)
export(tpm_hsmm2)
export(tpm_ihsmm)
export(tpm_p)
export(tpm_phsmm)
export(tpm_phsmm2)
export(tpm_thinned)
export(viterbi)
export(viterbi_g)
export(viterbi_p)
export(zero_inflate)
import(RTMB)
importFrom(MASS,ginv)
importFrom(RTMB,REPORT)
importFrom(RTMB,dnorm)
importFrom(RTMB,eigen)
importFrom(RTMBdist,dgamma2)
importFrom(RTMBdist,dskewnorm)
importFrom(RTMBdist,dvm)
importFrom(RTMBdist,dwrpcauchy)
importFrom(RTMBdist,mcreport)
importFrom(RTMBdist,pgamma2)
importFrom(RTMBdist,pskewnorm)
importFrom(RTMBdist,pvm)
importFrom(RTMBdist,qgamma2)
importFrom(RTMBdist,qskewnorm)
importFrom(RTMBdist,rgamma2)
importFrom(RTMBdist,rskewnorm)
importFrom(RTMBdist,rvm)
importFrom(RTMBdist,rwrpcauchy)
importFrom(Rcpp,sourceCpp)
importFrom(graphics,abline)
importFrom(graphics,hist)
importFrom(graphics,lines)
importFrom(graphics,par)
importFrom(mgcv,cSplineDes)
importFrom(mgcv,gam)
importFrom(mgcv,predict.gam)
importFrom(mgcv,rmvn)
importFrom(mgcv,s)
importFrom(numDeriv,jacobian)
importFrom(splines2,bSpline)
importFrom(stats,AIC)
importFrom(stats,BIC)
importFrom(stats,acf)
importFrom(stats,na.omit)
importFrom(stats,na.pass)
importFrom(stats,qqnorm)
importFrom(stats,runif)
importFrom(stats,update)
importFrom(utils,modifyList)
useDynLib(LaMa, .registration=TRUE)
