#' T-Rex Movement Data
#'
#' @description
#' Hourly step lengths and turning angles of a Tyrannosaurus rex, living 66 million years ago.
#'
#' @format A data frame with 10.000 rows and 4 variables:
#' \describe{
#'   \item{tod}{time of day variable ranging from 1 to 24}
#'   \item{step}{hourly step lengths in kilometres}
#'   \item{angle}{hourly turning angles in radians}
#'   \item{state}{hidden state variable}
#' }
#' @source Generated for example purposes.
"trex"

#' Loch Ness Monster Acceleration Data
#'
#' @description
#' A small group of researchers managed to put an accelerometer on the Loch Ness Monster and collected data for a few days. 
#' Now we have a data set of the overall dynamic body acceleration (ODBA) of the creature.
#' 
#' @format A data frame with 5.000 rows and 3 variables:
#' \describe{
#'   \item{ODBA}{overall dynamci body acceleration}
#'   \item{logODBA}{logarithm of overall dynamic body acceleration}
#'   \item{state}{hidden state variable}
#' }
#' @source Generated for example purposes.
"nessi"